#!/bin/bash

if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

. /etc/os-release

if [ "$ID_LIKE" != "debian" ]; then
  echo "Installer can run only on debian like system. Exiting." 
  exit 1 
fi

if [ "$ID" == "ubuntu" ]; then
  echo "Ubuntu found"
  . /etc/lsb-release
elif [ "$ID" == "raspbian" ]; then
  echo "Raspbian found"
else
  echo "Unsupported Debian version - it might or might not work"
fi
#. /etc/lsb-release

CWD=`pwd`
ROOT_DIR=/opt/lumipicker
ACTUAL_USER=`who am i | awk '{print $1}'`
USER_HOME=`getent passwd $ACTUAL_USER | cut -d: -f6`
BACKEND_ROOT_DIR=${ROOT_DIR}/backend
FRONTEND_ROOT_DIR=${ROOT_DIR}/frontend
CONFIG_DIR=${USER_HOME}/.lumipicker
DATA_DIR=${USER_HOME}/lumipicker


mkdir -p ${BACKEND_ROOT_DIR} ${FRONTEND_ROOT_DIR} ${CONFIG_DIR} ${DATA_DIR}
chown ${ACTUAL_USER}:${ACTUAL_USER} ${ROOT_DIR} ${BACKEND_ROOT_DIR} ${FRONTEND_ROOT_DIR} ${CONFIG_DIR} ${DATA_DIR}



echo "Installing GPhoto2"
pushd /tmp
  wget https://raw.githubusercontent.com/gonzalo/gphoto2-updater/master/gphoto2-updater.sh 
  chmod +x gphoto2-updater.sh 
  sudo ./gphoto2-updater.sh -d
popd

echo "Installing dependencies"
apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv 0C49F3730359A14518585931BC711F9BA15703C6
if [ "$ID" == "ubuntu" ]; then
  echo "deb [ arch=amd64 ] http://repo.mongodb.org/apt/ubuntu ${DISTRIB_CODENAME}/mongodb-org/3.4 multiverse" | tee /etc/apt/sources.list.d/mongodb-org-3.4.list
else 
  if [ "$VERSION_ID" == "8" ]; then
    echo "deb http://repo.mongodb.org/apt/debian jessie/mongodb-org/3.4 main" | tee /etc/apt/sources.list.d/mongodb-org-3.4.list
  elif [ "$VERSION_ID" == "7" ]; then
    echo "deb http://repo.mongodb.org/apt/debian wheezy/mongodb-org/3.4 main" | tee /etc/apt/sources.list.d/mongodb-org-3.4.list
  else 
    echo "Unsupported version of Debian [$VERSION_ID]"
  fi
fi;

apt-get update
apt-get install build-essential python-dev apache2 mongodb-org-server python2.7 libapache2-mod-wsgi python-pip dcraw libjpeg-dev



sudo -s -u ${ACTUAL_USER}<<EOF

echo "Installing python modules into the virtualenv"
pip install virtualenv

pushd ${BACKEND_ROOT_DIR}
  virtualenv -p /usr/bin/python2.7 venv
  source venv/bin/activate
    echo $VIRTUAL_ENV
    ln -sf /usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH`/libz.so ./venv/lib/
    ln -sf /usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH`/libfreetype.so ./venv/lib/
    ln -sf /usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH`/libjpeg.so ./venv/lib/
    pip install -e ${CWD}/lumipicker-backend 
    pip install --no-cache-dir -I pillow
  deactivate
  cp ${CWD}/lumipicker-backend/misc/lumipicker-backend.wsgi ./
popd

EOF

echo "Configuring Apache"
cp ${CWD}/lumipicker-backend/misc/lumipicker-backend.apache.conf /tmp/
sed -i "s#__USER__#$ACTUAL_USER#g" /tmp/lumipicker-backend.apache.conf
sed -i "s#__BACKEND_ROOTDIR__#$BACKEND_ROOT_DIR#g" /tmp/lumipicker-backend.apache.conf
sed -i "s#__FRONTEND_ROOTDIR__#$FRONTEND_ROOT_DIR#g" /tmp/lumipicker-backend.apache.conf
sudo cp /tmp/lumipicker-backend.apache.conf /etc/apache2/sites-enabled/

sudo service apache2 restart

echo "Creating backend config"
cp ${CWD}/lumipicker-backend/misc/config.yml /tmp/
sed -i "s#__DATAPATH__#$DATA_DIR#g" /tmp/config.yml

sudo -s -u ${ACTUAL_USER}<<EOF
  cp /tmp/config.yml ${CONFIG_DIR}/
  echo "Creating frontend"
  cp -r ${CWD}/lumipicker-frontend/app/export/* ${FRONTEND_ROOT_DIR}/
EOF

echo "Starting MongoDB"
service mongod start

echo "Instalation complete"
echo "Open browser on localhost:4000"
