#!/usr/bin/env python
import sys,os
import yaml
from bson.json_util import dumps

from flask import Flask, jsonify, request, send_file
from flask_restful import Resource, Api

from capture_service import CaptureService
from mongo_model import MongoModel


with open("%s/.lumipicker/config.yml"%(os.path.expanduser('~')), 'r') as ymlfile:
  cfg = yaml.load(ymlfile)


app = Flask(__name__)
api = Api(app)
app.debug = True

@app.after_request
def after_request(response):
  response.headers.add('Access-Control-Allow-Origin', '*')
  response.headers.add('Access-Control-Allow-Headers', 'Content-Type,Authorization')
  response.headers.add('Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE')
  return response
    

class RestCameraPreview(Resource):
    #TODO
    def __init__(self, service):
        self.service = service

    def get(self):
      print "Requesting preview"
      camera = request.args.get('camera')
      print camera
      return self.service.get_preview(camera)

class RestCaptureProject(Resource):
    #TODO
    def __init__(self, service):
        self.service = service

    def get(self):
      """
      Get current project status with updates, messages, countdown
      Basically physical bind for frontend
      'Whatever is on capture screen'
      """
      #print "Status requested"
      status = self.service.get_status()
      #print "Sending status"
      return status

    def post(self):
      """
      Control project setup with parameter settings, etc.
      """
      data = request.get_json(force=True)
      self.service.set_config(data)
      return self.service.get_status()

    def put(self):
      """
      Start the capturing process
      """
      print "Start requested"
      data = request.get_json(force=True)
      print "Params: ", data
      if 'description' in data.keys():
        desc = data['description']
      else:
        desc = None
      print data
      project_id = self.service.start_project(name=data["name"], cameras=data["cameras"], description=desc, 
      capture_interval=data['interval'], capture_start=data['capture_start'], 
      capture_end=data['capture_end'], ncaptures=data['ncaptures'], seconds_to_start=data['seconds_to_start']
      )
      print "Started"
      return {"id": project_id}
      

    def delete(self):
      """
      Stop the capturing process
      """
      print "Stop requested"
      self.service.stop_project()
      print "Stopped"


PROJECT_MAIN_FOLDER=cfg["datadir"]
if "username" in cfg["db"]:
  Model = MongoModel(cfg["db"]["name"], cfg["db"]["host"], cfg["db"]["port"], cfg["db"]["username"], cfg["db"]["password"])
else:
  Model = MongoModel(cfg["db"]["name"], cfg["db"]["host"], cfg["db"]["port"])
capture_service = CaptureService(PROJECT_MAIN_FOLDER, Model)
    
api.add_resource(RestCaptureProject, '/project-capture', resource_class_kwargs={ 'service': capture_service })
api.add_resource(RestCameraPreview, '/preview', resource_class_kwargs={ 'service': capture_service })


if __name__ == "__main__":  
  app.run(debug=False,host='0.0.0.0')






#~ if len(sys.argv) == 2 and sys.argv[1] == "--daemon":
  #~ print "Starting daemon"
  #~ from pwd import getpwnam  
  #~ syslog = open('/tmp/photo.log', 'w+')
  #~ 
  #~ pidfile=daemon.pidfile.PIDLockFile('/tmp/photo-backend.pid')
  #~ context = daemon.DaemonContext(working_directory="/home/pi/photo/backend",stdout=syslog, stderr=syslog, uid=getpwnam('pi').pw_uid, pidfile=pidfile)
  #~ with context:
    #~ main(debug=False)
  #~ print "Daemon exiting"
#~ else:
  #~ main()
  

  
  
  
