#!/usr/bin/env python

import utils
import threading
import os
import Queue

from camera_control import CameraControl
from project_worker import ProjectWorker
import image_analysis


class CaptureService(object):
  """
  Persistent service
  maintaining all communication with frontend
  keeping track of running captures
  """
  def __init__(self, projects_root, Model,  debug=True):
    print "Init"
    utils.mkdir_p(projects_root)
    self.projects_root = projects_root
    self.status_lock = threading.Lock()
    self.capture_queue = Queue.Queue()
    self.message_queue = Queue.Queue()
    self.Model = Model
    self.project_worker = None
    #BLOCK TOREMOVE
    #if not debug or os.environ.get("WERKZEUG_RUN_MAIN") == "true":
    self.camera_control = CameraControl(self.message_queue)
    self.camera_control.list_cameras()
    ##
    self.reset_status()
    self.cameras = {}
    


  def reset_status(self):
    self.status = {"cameras": [], "project": {"started": False,"waiting": False, "finished": False,"waiting": False, "cameras": {}}, "messages": []}

  def __enter__(self):
    print "Entering CaptureService"
    return self

  def __exit__(self, exc_type, exc_value, traceback):
    print "Cleaning up CaptureService"
    #if self.camera_control:
    #  self.camera_control.__exit__()


  def get_status(self):
    
    if self.status['project']["finished"]:
      self.reset_status()
      
    if self.project_worker:
      self.status['project']["started"] = self.project_worker.project_running;
    #print "Status 1"
    
    self.status["cameras"] = self.get_cameras(self.status['project']['started'] or self.status['project']["waiting"])
    #else:
    #  print "Started project, assuming cameras online"
    #print "Status 2"
    while not self.capture_queue.empty():
      d = self.capture_queue.get()
      if 'camera' in d.keys():
        camera = d['camera']
        if camera not in self.status['project']['cameras'].keys():
          self.status['project']['cameras'][camera] = {}
        #print "Capture Data: ", d
        if "capture" in d['data']:
          if d['data']["capture"]:
            self.status['project']['cameras'][camera]["thumbnail"] = None
            self.status['project']['cameras'][camera]["histogram"] = None
            self.status['project']['cameras'][camera]["luminance"]  = None
          else:
            self.status['project']['cameras'][camera]["thumbnail"] = -1
            self.status['project']['cameras'][camera]["histogram"] = -1
            self.status['project']['cameras'][camera]["luminance"]  = -1
        if "thumbnail" in d['data']:
          self.status['project']['cameras'][camera]["thumbnail"] = d['data']['thumbnail']['data']
        if "histogram" in d['data']:
          self.status['project']['cameras'][camera]["histogram"] = d['data']['histogram']['data']
        if "luminance" in d['data']:
          self.status['project']['cameras'][camera]["luminance"] = d['data']['luminance']['data']
        if "metadata" in d['data']:
          self.status['project']['cameras'][camera]["metadata"] = d['data']['metadata']
        
          
      if "model" in d['data']:
        self.Model.capture_save(d['data']['model']) 
      
      if "ncaptures_countdown" in d['data']:
        self.status['project']["ncaptures_countdown"] = d['data']["ncaptures_countdown"]
        
      if "waiting" in d['data']:
        print "Waiting status", d['data']["waiting"]
        self.status['project']["waiting"] = d['data']["waiting"]
        
      if "finished" in d['data']:
        print "Finished true"
        self.status['project']["finished"] = True;
        
        
      if "ncaptures_todo" in d['data']:
        self.status['project']["ncaptures_todo"] = d['data']["ncaptures_todo"]
    #print "Status 3"      
    messages = []
    while not self.message_queue.empty():
      m = self.message_queue.get()
      print "Message: ", m 
      messages += [m]
    self.status["messages"] = messages
    return self.status


  def set_config(self, config):
    print config
    if "cameras" in config:
      for camera,values in config["cameras"].iteritems():
        print camera, values
        for k,v in values.iteritems():
          self.camera_control.set_camera_config(camera, str(k), str(v))
          
        #self.camera_control.set_camera_config()
    
  def get_cameras(self, started):
    """

    """
    #print "GetCameras: 1"
    if not started:
      newcams = self.camera_control.list_cameras()
      #print "GetCameras: 2"
      for k,v in newcams.iteritems():
        cam_db = self.Model.camera_get({"name":k})
        if not cam_db:
          cam_model = list(utils.gen_dict_extract("(cameramodel)",v["config"]))[0][1]
          cam_manu = list(utils.gen_dict_extract("(manufacturer)",v["config"]))[0][1]
          self.Model.camera_save({"name":k, "model": cam_model, "manufacturer": cam_manu, "K_c": 1.0})      
      self.cameras = {k:v["config"] for k,v in newcams.iteritems()}
    else:
      print "Project running - assuming no camera change"
    return self.cameras


  def start_project(self, name=None, cameras=None, description=None, capture_interval=None, capture_start=None, capture_end=None, ncaptures=None, seconds_to_start=None):
    if not self.project_worker or not self.project_worker.project_running:
      self.reset_status()
      actual_project_config = {"name":name, "cameras":cameras,"description":description,"capture_interval":int(capture_interval),"capture_start":capture_start,"capture_end":capture_end,"ncaptures":int(ncaptures)}
      project_id = self.Model.project_save(actual_project_config)
      project_root = "%s/%s/"%(self.projects_root, project_id)
      utils.mkdir_p(project_root)
      
      K_c = {camera: float(self.Model.camera_get({"name":camera})["K_c"]) for camera in cameras}
      
      self.project_worker = ProjectWorker(self.capture_queue,self.message_queue, self.camera_control, K_c, project_root, project_id, actual_project_config, seconds_to_start)
      self.project_worker.start()
      self.status['project']['config'] = actual_project_config
      self.status['project']['projectid'] = project_id
      self.status['project']['started'] = True;
      return project_id
    else:
      self.message_queue.put({"message": "Another project already running", "type": "danger"})
    
  def stop_project(self):
    if self.project_worker:
      self.project_worker.terminate_project = True
      self.project_worker.join()
    self.status['project']['started'] = False;


  def get_updates(self, key):
    ret = {}
    if self.capture_queue.qsize() > 0:
      fname, meta, histogram, luminance = self.capture_queue.get()
      ret['img'] = self.get_image("%s.jpeg"%(fname))
      ret['meta'] = meta
      ret['histogram'] = self.get_image(histogram, 256, fmt='PNG')
      ret['luminance'] = self.get_image(luminance, 256, fmt='PNG')
      status = 200
    else:
      status = 304
    return ret, status
    
  def get_preview(self, camera):
    raw_path = "/tmp/preview.raw"
    jpeg_path = "/tmp/preview.jpeg"
    if (self.camera_control.capture_image(camera, raw_path)):
      image_analysis.thumbnail(raw_path, jpeg_path)
      b64 = image_analysis.get_image(jpeg_path)
      os.unlink(raw_path)
      os.unlink(jpeg_path)
      return {"data": b64}
    else:
      self.message_queue.put({"message": "Error in preview capture [%s]"%(camera), "type": "danger"})
      return {"data": "-1"}
