import {inject, bindable} from 'aurelia-framework';
import {NotificationService} from 'aurelia-notify';
import {RestAnalyzeControlAPI} from './control-api';
import {EventAggregator} from 'aurelia-event-aggregator';


@inject(NotificationService, RestAnalyzeControlAPI, EventAggregator)
export class Analyze {

  constructor(private notificationService : NotificationService, private analyze_api : RestAnalyzeControlAPI, private ea: EventAggregator) {}
  //message = "Data:"+this.config.projectUri;
  luminance_img : any;
  luminance_colorbar : any;
  luminace_lut: any;
  histogram : any;
  line_histogram : any;
  img : any;
  metadata: Object;
  stats: Object;
  line_stats: Object;
  cropData: Object;
  projectId: any;
  projectCamera: any;
  photoId: any;
  baseLoaded = false;
  statsLoading = false;

  pdf: String;
  luminance_pdf: String;
  pointValue: number;

  pointCoords: Object;
  lineCoords: Object;
  rectangleBounds = Object;

  loading: Object;
  uiLock: boolean;
  calibrationEnabled: boolean;
  calibrationValue: any;


  created(){

    //this.analyze_api.analyze
    this.loading = Object();
    this.loading["metadata"] = true;
    this.loading["img"] = true;
    this.loading["luminance"] = true;
    this.loading["stats"] = true;
    this.uiLock = true;
    this.calibrationEnabled = false;

  }
  activate(params) {
      console.log(params);
      this.projectId = params.projectid;
      this.projectCamera = params.camera;
      this.photoId = params.photoid;
      this.getRectangleStats(null);


      //this.notificationService.info(`Welcome ${this.photoid}`);
  }


  handleError(e, msg=null){
      this.lock(false);
      this.notificationService.danger("Error while fetching capture data from server.")
      console.error(e);
  }

  getRectangleStats(coords){
      if (coords){
        this.lock(true);
        this.statsLoading = true;
        this.pointCoords = null;
        this.lineCoords = null;
        this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'stats', coords)
        .then(d =>
          {
              this.stats = d;
              this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'histogram', coords)
              .then(d =>
                {
                  this.histogram = d;
                  this.baseLoaded = true;
                  this.statsLoading = false;
                  this.notificationService.success("Rectangle updated")
                  this.lock(false);
              }, e => {this.handleError(e)});
          },e => {this.handleError(e)});
      }
      else{
        //   this.loading["img"] = true;
          this.lock(true);
          this.statsLoading = true;
          return this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'metadata', coords)
            .then(d =>
                {
                    this.metadata = d;
                    //this.notificationService.info("Metadata loaded")
                    this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'img', coords)
                    .then(d =>
                      {
                          this.img = d;
                        //   this.loading["img"] = false;
                        this.notificationService.info("Capture loaded")
                          this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'luminance', coords)
                          .then(d =>
                            {
                                this.luminance_img = d;
                                this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'luminance_bar', coords)
                                .then(d =>
                                    {
                                        this.notificationService.info("Luminance loaded")
                                        this.luminance_colorbar = d["luminance_bar"];
                                        this.luminace_lut = d["lut"];
                                        this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'stats', coords)
                                        .then(d =>
                                          {

                                              this.stats = d;
                                              this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'histogram', coords)
                                              .then(d =>
                                                {
                                                  this.notificationService.info("Histogram computed")
                                                  this.histogram = d;
                                                  this.baseLoaded = true;
                                                  this.statsLoading = false;
                                                  this.lock(false);
                                                },e => {this.handleError(e)});
                                          },e => {this.handleError(e)});
                                    },e => {this.handleError(e)});
                            },e => {this.handleError(e)});
                    },e => {this.handleError(e)});
                },e => {this.handleError(e)});
        }
  }



  export(type){
      let coords = {}
      if (this.rectangleBounds){
        coords["rectangle"] = this.rectangleBounds
      }

      if (this.pointCoords){
          coords["point"] = this.pointCoords
      }

      if (this.lineCoords){
          coords["line"] = this.lineCoords
      }
      this.pdf = null;
      this.notificationService.info("Generating PDF")
      this.lock(true)
      let promise = this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, type, coords)
      .then(d =>
        {
            if (type == "pdf"){
                let pdf_blob = this.b64toBlob(d, 'application/pdf', 512);
                this.pdf = window.URL.createObjectURL(pdf_blob);
                this.notificationService.success("PDF created")
                this.lock(false)
            }
            else{
                let pdf_blob = this.b64toBlob(d, 'application/pdf', 512);
                this.luminance_pdf = window.URL.createObjectURL(pdf_blob);
                this.notificationService.success("PDF created")
                this.lock(false)
            }


        },e => {this.handleError(e)})


  }

  onGuilotineUpdate(d){

      this.rectangleBounds = d;
      if (this.baseLoaded){
          this.getRectangleStats(this.rectangleBounds);
          console.log("onGuilotineUpdate fired with", this.rectangleBounds);
      }
  }

  pointUpdated(point, value){

      let r = this.luminace_lut.r.indexOf(value[0]);
      let g = this.luminace_lut.g.indexOf(value[0]);
      let b = this.luminace_lut.b.indexOf(value[0]);
      console.log(point, value, r,g,b);
      let factor = 1.0/this.rectangleBounds['scale'];

      let x = Math.round(point.x*factor + this.rectangleBounds['x']*factor);
      let y = Math.round(point.y*factor + this.rectangleBounds['y']*factor);
      this.pointCoords = {"x": x, "y": y};
      this.lock(true);
      return this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'point', [x, y])
        .then(d =>
            {
                this.pointValue = d;
                this.lock(false);
            },e => {this.handleError(e)});
  }

  lineUpdated(line){
      console.log(line)
      let factor = 1.0/this.rectangleBounds["scale"]
      let abs_coords = {ax: (this.rectangleBounds["x"]+line.a.x)*factor, ay: (this.rectangleBounds["y"]+line.a.y)*factor,
                        bx: (this.rectangleBounds["x"]+line.b.x)*factor, by: (this.rectangleBounds["y"]+line.b.y)*factor}
      console.log("Coords", abs_coords)
      this.lineCoords = abs_coords;
      this.lock(true);
      return this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'line', abs_coords)
      .then(d =>
        {
            console.log(d["line_stats"]);
            this.line_stats = d["line_stats"];
            this.line_histogram = d["line_histogram"];
            this.lock(false);
            this.notificationService.success("Line analysis done")
        },e => {this.handleError(e)});

  }

  lock(lock){
      this.uiLock = lock;
      this.ea.publish('uiLock', lock);
  }

  enable_calibration(){

      this.calibrationEnabled = !this.calibrationEnabled;
  }


   isNumber(n){
    return Number(n) === n
   }

  get calibrateButtonCondition() {
      //return this.isNumber(this.calibrationValue)
      return true
  }

  calibrate(){
      this.lock(true);
      return this.analyze_api.analyze(this.projectId, this.projectCamera, this.photoId, 'calibrate', this.pointCoords, this.calibrationValue)
      .then(d =>
        {
            this.lock(false);
            this.getRectangleStats(null);
        },e => {this.handleError(e)});
  }

  b64toBlob(b64Data, contentType, sliceSize) {
  contentType = contentType || '';
  sliceSize = sliceSize || 512;

  var byteCharacters = atob(b64Data);
  var byteArrays = [];

  for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
    var slice = byteCharacters.slice(offset, offset + sliceSize);

    var byteNumbers = new Array(slice.length);
    for (var i = 0; i < slice.length; i++) {
      byteNumbers[i] = slice.charCodeAt(i);
    }

    var byteArray = new Uint8Array(byteNumbers);

    byteArrays.push(byteArray);
  }

  var blob = new Blob(byteArrays, {type: contentType});
  return blob;
}


}
