import {inject, customAttribute, bindable, Parent} from 'aurelia-framework';
import {EventAggregator} from 'aurelia-event-aggregator';
import 'jquery';

//import 'guillotine';
//import 'guillotine/css/jquery.guillotine.css!';
//import * as Moment from 'moment';

//import 'jquery-datetimepicker';
//import 'jquery-datetimepicker/jquery.datetimepicker.css!';

@customAttribute('luminance-measurement')
@inject(Element, EventAggregator)
export class LuminanceMeasurement {
  @bindable onChange: any;
  @bindable onLineUpdated: any;
  @bindable onPointSelected: any;
  ea: EventAggregator;
  msgSubscription: any;
  lockSubscription: any;
  img: any;
  line: {a: {x: number, y: number}, b: {x: number, y: number}};
  point: {x: number, y: number};
  imgProperties : {x: number, y: number, w: number, h: number, scale: number, angle: number};
  uiLock : boolean;
  constructor(private element, private eventAggregator) {
      this.ea = eventAggregator;
      this.resetArtifacts();
      this.uiLock = true;
  }

  getMousePos(canvas, evt) {
    let rect = canvas.getBoundingClientRect();

    return {
        x: Math.round(evt.clientX - rect.left),
        y: Math.round(evt.clientY - rect.top)
    };
}

  resetArtifacts(){
      this.line = {
          a:  {x: null, y: null},
          b:  {x: null, y: null},
      };
      this.point = {x: null, y: null};
  }

  drawImage() {
    let parent = this;
    let canvas = ($('#canvas-container canvas') as any)[0];
    canvas.width = parent.imgProperties.w;
    canvas.height = parent.imgProperties.h;

    console.log(canvas.width, canvas.height, parent.imgProperties)
    let context = canvas.getContext("2d")
    //context.drawImage(image, 0, 0);
    context.save()
    let factor = 1./parent.imgProperties.scale
    context.drawImage(this.img, Math.round(parent.imgProperties.x*factor), Math.round(parent.imgProperties.y*factor), Math.round(parent.imgProperties.w*factor), Math.round(parent.imgProperties.h*factor), 0, 0, canvas.width, canvas.height);
    context.scale(parent.imgProperties.scale, parent.imgProperties.scale);


    context.restore();
    if (parent.point.x){ //draw point

        context.beginPath();
        context.arc(parent.point.x,parent.point.y,5,0,2*Math.PI);
        context.stroke();

    }

    if (parent.line.a.x && parent.line.b.x){ //draw line
        context.moveTo(parent.line.a.x, parent.line.a.y);
        context.lineTo(parent.line.b.x, parent.line.b.y);
        context.stroke();
    }


  }

  attached() {

      let el = ($(this.element) as any);
      let parent = this;
      el.one('load', function(){

          let ar = el.width()/el.height();
          let height = 400;
          let width = height*ar;

          //console.log(el.width, ar, guill_width, guill_height);
          ($('#canvas-container') as any).css({'width': width, 'height': height})

          parent.img = el[0];
          parent.msgSubscription = parent.eventAggregator.subscribe('coordsUpdated', d => {console.log("Gill says", d); parent.resetArtifacts(); parent.imgProperties = d; parent.drawImage()});
          parent.lockSubscription = parent.eventAggregator.subscribe('uiLock', d => {console.log("Lumik: Parent says lock is ", d);parent.uiLock = d})
          let canvas = ($('#canvas-container canvas') as any)

          canvas.on('click', function(e){
              if (parent.uiLock){
                  return false;
              }
              parent.resetArtifacts();
              let pos = parent.getMousePos(canvas[0], e)

              console.log("Mousedown", pos, canvas[0].getContext("2d").getImageData(pos.x, pos.y, 1, 1).data)
              parent.point = pos;
              parent.onPointSelected({'point': parent.point, 'value': canvas[0].getContext("2d").getImageData(pos.x, pos.y, 1, 1).data})
              parent.drawImage();
              return false;

          });

          canvas.on('contextmenu', function(e){
              if (parent.uiLock){
                  return false;
              }
              let pos = parent.getMousePos(canvas[0], e)
              if (parent.line.a.x && parent.line.b.x)
              {
                  parent.resetArtifacts();
              }
              if (parent.line.a.x){
                  console.log("Setting point B", pos)
                  parent.line.b = pos
                  parent.onLineUpdated({'line': parent.line})
                  parent.drawImage();
              }else{
                  console.log("Setting point A", pos)
                  parent.line.a = pos
                  parent.drawImage();
              }

              return false;
          });
          parent.ea.publish('luminaceReady', true);
        });
      if (el.prop('complete')) el.trigger('load');


  }

  detached() {
    // remove it in here if possible!
    this.msgSubscription.dispose()
  }

  lineDistance(x, y, x0, y0){
      return Math.sqrt((x -= x0) * x + (y -= y0) * y);
  }

  drawLine(elemName, line) {
    var angle = Math.atan2(line.b.y - line.a.y, line.b.x - line.a.x) * 180 / Math.PI;
    var distance = this.lineDistance(line.a.x, line.a.y, line.b.x, line.b.y);

    console.log(line.a);

    // Set Angle
    ($(elemName)as any).css('transform', 'rotate(' + angle + 'deg)');

    // Set Width
    ($(elemName)as any).css('width', distance + 'px');

    // Set Position
    ($(elemName)as any).css('position', 'absolute');
    if(line.b.x < line.a.x) {
      console.log("version A");
      if(line.b.y < line.a.y) {
          console.log("Ay > By");
          ($(elemName)as any).offset({top: line.b.y, left: line.b.x});
      }else{
          console.log("Ay < By");
          ($(elemName)as any).offset({top: line.a.y, left: line.b.x});
      }

    } else {

        console.log("version B");
        if(line.b.y < line.a.y) {
                ($(elemName)as any).offset({top: line.b.y, left: line.a.x});
        }
        else{
            ($(elemName)as any).offset({top: line.a.y, left: line.a.x});
        }

  }
  }
}






function fireEvent(element, name) {
  var event = document.createEvent('Event');
  event.initEvent(name, true, true);
  element.dispatchEvent(event);
}
