import {inject} from 'aurelia-framework';
import {Router} from 'aurelia-router';
import {EventAggregator} from 'aurelia-event-aggregator';
import {RestCaptureContolAPI, RestPreviewAPI, RestCalibrateAPI} from './control-api';
import {areEqual} from './utility';
import {Status} from './containers/status';
import {ProjectConfig} from './containers/project-config';
import {NotificationService} from 'aurelia-notify';
import * as moment from 'moment'





@inject(RestCaptureContolAPI, Status, ProjectConfig, NotificationService, RestPreviewAPI, RestCalibrateAPI, Router, EventAggregator)
export class ControlForm {
  constructor(  private api: RestCaptureContolAPI,
                private status : Status,
                private project: ProjectConfig,
                private notificationService : NotificationService,
                private prview_api : RestPreviewAPI,
                private calibrate_api : RestCalibrateAPI,
                private router: Router,
                private ea: EventAggregator
            ) {}
  routeConfig;
  control;
  timer : any;
  ws_id = false;
  counter = 0;
  started = false;
  selectedCameras = [];
  selectedCameraConfig;
  reStart = true;
  useCustomStart = false;
  backendConnected = false;
  componentAttached = false;

  attached(){
      this.componentAttached = true;
      this.mainLoop()
  }

  detached(){
      console.log("detaching controller")
      this.componentAttached = false
      clearInterval(this.timer);
  }

  isInteger(x) {
    return x % 1 === 0;
  }


  get startButtonCondition() {
    return this.project.cameras && this.project.cameras.length > 0 &&
           this.project.name &&
           this.project.interval && this.isInteger(this.project.interval) &&
           this.project.ncaptures && this.isInteger(this.project.ncaptures) &&
           this.project.interval > 25 && !this.status.project.started;
  }

  setCameraTime(camera){
      let d = new Date();
      console.log("Setting time", Math.round(d.getTime() / 1000))
      this.setCameraConfig(camera, "(datetime)",  (Math.round(d.getTime() / 1000)).toString())
  }

  setCameraConfig(camera, name, value){
      let cam_config = Object();
      let config = Object();
      cam_config[name] = value;
      let data = Object();
      data.cameras = Object();
      data.cameras[camera] = cam_config;
      this.setProjectConfig(data);
  }

  setProjectConfig(dict){ //send configuration to backend
      this.api.setConfig(dict).then(status => {this.handleGetStatus(JSON.parse(status.response))})
  }

  cameraSelected(camera){
      if (this.project.cameras.indexOf(camera) === -1){ //adding camera to project
          this.project.cameras.push(camera)
          if (!this.selectedCameraConfig){
              this.getCameraConfig(camera);
              this.selectedCameraConfig = camera;
          }
      }
      else{ //remove camera from project
          this.project.cameras.splice(this.project.cameras.indexOf(camera), 1)
          if (this.project.cameras.length == 0){
              this.selectedCameraConfig = null;
          }
          else if(this.selectedCameraConfig == camera){
              this.getCameraConfig(this.project.cameras[0]);
              this.selectedCameraConfig = this.project.cameras[0];
          }
      }
      return true;
  }

  start() {
    console.log(this.project.description, this.project.cameras, this.project.interval);
        if (!this.project.capture_start) {
            console.log("Setting NOW")
            this.project.capture_start = new Date();
            this.project.seconds_to_start = 0;
        }
        else
        {
            console.log("Seeting Moment", this.project.capture_start)
            var now = moment(new Date()); //todays date
            var start = this.project.capture_start; // another date
            var duration = moment.duration(start.diff(now));
            var seconds_to_start = duration.asSeconds();
            this.project.seconds_to_start = seconds_to_start;
            console.log("Seconds to start:", seconds_to_start)
            this.project.capture_start = this.project.capture_start.toDate();
    }



    this.project.capture_end = new Date()
    this.project.capture_end.setSeconds(this.project.capture_start.getSeconds() + this.project.interval*this.project.ncaptures);

    this.project.preview = null;


    this.api.start(this.project).then(
      data => {
        this.ws_id = data.content.id;
        this.status.project.started = true
        this.reStart = false
      }
    )
  }



  get stopButtonCondition() {
    if (this.status.project){
        return this.status.project.started;
    }
    else{
        return false;
    }
  }

  startDate(val){
     this.useCustomStart = !val;
     return true
  }

  stop(){
    this.api.stop(this.project).then(a => this.status.project.started = false)
    this.project.capture_start = "";

  }



  mainLoop(){
        this.timer = setInterval(a => {this.mainLoopAction()}, 1000);
  }

  mainLoopAction()
  {
    if (this.counter%5 == 0){
        this.getStatus();
        this.counter = 0;
        console.log(this.project.cameras);
    }
    this.counter++;
    if (this.status.project && this.status.project.started)
    {
        if (this.status.project.ncaptures_countdown > 0){
            this.status.project.ncaptures_countdown -= 1;
            this.status.project.ncaptures_progress = (this.status.project.ncaptures_countdown/this.status.project.config.capture_interval)*100
        }
    }
  }

  recursiveGetProperty(obj, lookup) {
      for (let property in obj) {

          if (property.indexOf(lookup) !== -1) {
              //console.log(obj[property]);
              return obj[property];
          } else if (obj[property] instanceof Object) {
              var ret = this.recursiveGetProperty(obj[property], lookup);
              if (ret){
                return ret;
              }
          }
      }
  }

  showNotification(message){
      console.log(message);
      if (message.type == "danger") {
          this.notificationService.danger(`${message.message}`)
      }
      else if (message.type == "success") {
          this.notificationService.success(`${message.message}`)
      }
      else if (message.type == "info") {
          this.notificationService.info(`${message.message}`)
      }
      else if (message.type == "warning") {
          this.notificationService.warning(`${message.message}`)
      }
      else if (message.type == "debug") {
          console.debug(message.message);
      }
  }

  handleGetStatus(status){
      console.log(status);
      this.ea.publish('status', status);
      if (status.messages){
          for (let message of status.messages) {
            this.showNotification(message);
          }
      }

      if (status.project.finished){
        console.log("Project Finished - cleaning up")
        this.concludeProject()
      }

      if (status.project){
          this.status.project = status.project;
      }

      if (status.cameras){
          this.status.cameras = status.cameras;
          if (this.selectedCameraConfig){
              if(Object.keys(this.status.cameras).indexOf(this.selectedCameraConfig) === -1){
                  this.cameraSelected(this.selectedCameraConfig);
              }
              else{
                  this.getCameraConfig(this.selectedCameraConfig);
              }
          }
      }

      if(this.status.project.waiting){
          let now = moment();
          let start = now.add(this.status.project.ncaptures_countdown, 'seconds')
          console.log(start, this.status.project.ncaptures_countdown);
          this.status.project.start_text = start.fromNow();
          //this.status.project.wait_countdown = moment.
      }




      if (status.project.started && this.reStart){
          console.log("Reinit app")

          this.project.name = status.project.config.name

          this.project.description = status.project.config.description
          this.project.interval = status.project.config.capture_interval
          this.project.ncaptures = status.project.config.ncaptures
          this.project.capture_start = status.project.config.capture_start
          for (var camera of status.project.config.cameras){
              this.cameraSelected(camera)
          }
          this.reStart = false
      }
  }

  concludeProject(){
      this.status = new Status;
      this.project = new ProjectConfig;
      this.selectedCameraConfig = null;
  }
  getCameraConfig(camera){
          this.selectedCameraConfig = camera;
          this.status.project.camera_iso = this.recursiveGetProperty(this.status.cameras[camera], 'iso');
          this.status.project.camera_aperture = this.recursiveGetProperty(this.status.cameras[camera], 'aperture');
          this.status.project.camera_shutter = this.recursiveGetProperty(this.status.cameras[camera], 'shutterspeed');
          this.status.project.camera_battery = this.recursiveGetProperty(this.status.cameras[camera], 'battery');
          this.status.project.camera_lens = this.recursiveGetProperty(this.status.cameras[camera], 'lensname');
          let time = this.recursiveGetProperty(this.status.cameras[camera], '(datetime)');
          if (!time){
              time = this.recursiveGetProperty(this.status.cameras[camera], 'd034');
          }
          this.status.project.camera_time = time
          //console.log(time);
  }

  connectionStatus(connected){
      if (connected && !this.backendConnected){
            this.showNotification({"message": "Server connected", "type": "success"})
            this.backendConnected = true
      }

      if (!connected && this.backendConnected){
            this.showNotification({"message": "Server disconnected", "type": "danger"})
            this.backendConnected = false
      }

  }

  getStatus(){
      this.api.getStatus()
      .then(status => {this.connectionStatus(true); this.handleGetStatus(status)}, e => {this.connectionStatus(false)}
      );
  }

  preview(camera){
      this.project.previewRequesting = true;
      this.prview_api.preview(camera).then(content => {
          this.project.preview = content.data;
          this.project.previewRequesting = false;
          if(this.project.preview){
              if(this.project.preview == "-1"){
                  this.project.previewError = true;
              }
              else{
                  this.project.previewError = false;
              }
          }
      });
      console.log(camera)
  }


}
